<?php

namespace HCM\Util;

use Doctrine\ORM\EntityManager;

/**
 *  Classe construida com o proposito de organizar rotinas uteis para a manipulacao de entidades Doctrine
 * 
 */

class DoctrineEntityUtil {
    
    /**
     *  Controla entidades que, no momento da criacao, foram definidas como pendentes de 'persist'. Essa tratativa foi implementada considerando casos em que eh
     * necessario realizar determinados persists antes de outros em uma transaction
     * 
     */
    private $persistPendingEntities = [];

    /**
     *  Rotina que monta e retorna um objeto a partir de um array informado. Serão chamados métodos 'set' de acordo com as keys do array, passando os valores. Os métodos serão chamados da seguinte forma:
     *  '$newEntity->set[key]($value)' para cada item do array. O objeto será criado a partir do namespace informado.
     *
     *  @param  string      $entityRepository
     *  @param  Array       $fieldValues
     *  @param  bool        $setDefaultFields   = true [Util::setNewEntityDefaultFields]
     *  @param  bool        $persist            = true
     *  @param  bool        $flush              = true
     *  @param  bool        $pending*           = true [Entidade eh armazenada para posterior persist atraves de self::persistPending]
     * 
     *  *Implementacao pensada em casos que ha a criacao de entidades pai-filho ou mestre-historico. O Doctrine possui rotinas que determinam a ordem na qual devem ser
     * executados os inserts/updates no momento da chamada da rotina 'flush'. Essa ordem gera, para o cenario do sistema, inconsitencia em relacao ao tipo de entidades
     * citado. Por isso, em casos como o descrito, informar entidades 'pai' ou 'mestre' como 'pending' e persisti-las antes do 'flush' invocando a rotina self::persistPending
     *
     *  @return $newEntity [$entityRepository]
     */
    public function setNewEntity(
        string  $entityRepository,
        Array   $fieldValues,
        bool    $setDefaultFields   = false,
        bool    $persist            = false,
        bool    $flush              = false,
        bool    $pending            = false
    ) {
        $newEntity = new $entityRepository;
        
        if ($setDefaultFields) {
            $fieldValues = array_merge($fieldValues, $this->setNewEntityDefaultFields($setDefaultFields));
        }
        
        foreach ($fieldValues as $field => $value) {
            $setMethod = $this->buildSetEntityMethod($field);
            if (!method_exists($newEntity, $setMethod)) {
                continue;
            }
            
            $newEntity->$setMethod($value);
        }
        
        if ($persist) {
            Util::getEntityManager()->persist($newEntity);
        } else if ($pending) {
            $this->persistPendingEntities[] = $newEntity;
        }
        
        if ($flush) {
            Util::getEntityManager()->flush($newEntity);
        }
        
        return $newEntity;
    }
    
    /**
     *  Persiste as entidades definidas como 'pending' no momento de sua criacao com o metodo self::setNewEntity
     * 
     */
    public function persistPending() {
        foreach ($this->persistPendingEntities as $entity) {
            Util::getEntityManager()->persist($entity);
        }
        
        $this->persistPendingEntities = [];
    }
    
    /**
     *  Rotina que monta o método 'set' de um objeto, a partir de um nome.
     *  Esquema de funcionamento:
     *      ('nome') => 'set' . 'Nome'
     * 
     *  @param  string  $field
     * 
     *  @return string
     */
    private function buildSetEntityMethod(string $field): string {
        return 'set' . ucwords(strtolower($field));
    }
    
    /**
     *  Rotina que retorna um array com informações recorrentes das entidades do sistema
     * 
     *  @param  $entity
     * 
     *  @return Array
     */
    private function setNewEntityDefaultFields($entity) {
        $date = new \DateTime();
        
        $defaultFields = [
            'NRORG'             => Util::getHCMParameters()->getNrorgTrabalho(),
            'IDATIVO'           => 'S',
            'NRORGINCLUSAO'     => Util::getHCMParameters()->getNrorgTrabalho(),
            'DTINCLUSAO'        => $date,
            'CDOPERINCLUSAO'    => Util::getHCMParameters()->getCdoperador(),
            'NRORGULTATU'       => Util::getHCMParameters()->getNrorgTrabalho(),
            'DTULTATU'          => $date,
            'CDOPERULTATU'      => Util::getHCMParameters()->getCdoperador()
        ];
        
        return $defaultFields;
    }
    
} 